"""Functions for manipulating data."""
def truncate(string, char_limit):
    """truncate string to provided character limit"""
    length = char_limit - 3 # 3 for the ellipsis
    if len(string) > length:
        return string[:length] + "..."
    else:
        return string


"""A class for securely storing and retrieving environment variables"""
from cryptography.fernet import Fernet
import os

class SecureEnv:
    def __init__(self, key):
        self.cipher = Fernet(key)

    def decrypt(self, value):
        if self.check_env_exists(value):
            return self.cipher.decrypt(os.environ[value]).decode()
        else:
            print("Environment variable not found.")
            choice = input("Would you like to enter the value for the environment variable "+value+" now? (y/n): ")
            if choice.lower() == "y":
                value = input("Enter the value for the environment variable "+value+": ")
                self.add_env(value, self.cipher.encrypt(value.encode()))
                return value
            else:
                print("Cannot continue without the environment variable.")
                print("Exiting program.")
                exit()
    
    def encrypt(self, value):
        return self.cipher.encrypt(value.encode())

    def check_env_exists(self, env_name):
        return env_name in os.environ
    
    def add_env(self, env_name, value):
        os.environ[env_name] = self.encrypt(value)


"""A simple logger class for logging to the console and a file."""
from datetime import datetime
import os, json

class Logger:
    def __init__(self, appname, source="General"):
        self.source = source #which part of the app is logging
        self.appname = appname #name of the app
    
    #Logging functions
    def commit(self, message):
        """Logs a string to the console and a file."""
        log_text = message + "\n"
        print(log_text)    
        self.write_to_file(log_text)

    def datetime(self, message):
        """Logs a string with a timestamp to the console and a file."""
        log_text = str(datetime.now())+" - " + message +"\n"
        print(log_text)
        self.write_to_file(log_text)       

    #JSON functions
    def json(self, json_obj):
        """Logs a JSON object to the console and a file."""
        log_text = json.dumps(json_obj, indent=4)
        print(log_text)
        self.write_to_file(log_text)

    def save_json_to_own_file(self, json_obj, filename):
        """Saves a JSON object to a file. Overwrites the file if it exists."""
        self.pretty_print_json(json_obj)
        with open(filename, 'w') as outfile:
            json.dump(json_obj, outfile, indent=4)        

    def pretty_print_json(self, json_obj): #only prints to console
        """Prints a JSON object to the console with indentation."""
        print(json.dumps(json_obj, indent=4))

    #File and directory functions
    def write_to_file(self, message):
        """Writes a string to a file."""
        log_path = "c:\\windows\\temp\\"+self.appname+"\\"+self.source+".log"
        log_parent_path = "c:\\windows\\temp\\"+self.appname+"\\"
        if not self.check_file_exists(log_path):
            if not self.check_directory_exists(log_parent_path):
                self.create_directory(log_parent_path)
            self.create_file(log_path)
        log_file = open(log_path, "a")
        log_file.write(message)
        log_file.close()     

    def check_file_exists(self, path):
        """Checks if a file exists."""
        try:
            open(path, 'r')
            return True
        except:
            return False
        
    def check_directory_exists(self, path):
        """Checks if a directory exists."""
        try:
            os.listdir(path)
            return True
        except:
            return False    

    def create_directory(self, path):
        """Creates a directory."""
        try:
            os.mkdir(path)
            return True
        except:
            return False

    def create_file(self, path):
        """Creates a file."""
        try:
            open(path, 'w')
            return True
        except:
            return False